"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function renderItems(items, renderRowContents, rowHeight, el) {
  var viewportHeight = window.innerHeight;
  var elTop = el.getBoundingClientRect().top;
  var offset = -Math.ceil(Math.min(0, elTop) / rowHeight);
  var limit = Math.ceil(Math.min(viewportHeight, viewportHeight - elTop) / rowHeight);
  var itemsHtml = '';

  for (var i = offset, j = Math.min(items.length, offset + limit); i < j; ++i) {
    itemsHtml += "\n      <li class=\"windowed__item\"\n          style=\"position: absolute; width: 100%; transform: translateY(".concat(i * rowHeight, "px)\">\n        ").concat(renderRowContents(items[i]), "\n      </li>\n    ");
  }

  return itemsHtml;
}

function setElHeight(items, rowHeight, list) {
  list.style.height = "".concat(rowHeight * items.length, "px");
}

function optimiseHandler(handler) {
  var isTicking = false;
  return function () {
    if (!isTicking) {
      window.requestAnimationFrame(function () {
        handler();
        isTicking = false;
      });
      isTicking = true;
    }
  };
}

function windowed(options, el) {
  var _options$items = options.items,
      items = _options$items === void 0 ? [] : _options$items,
      rowHeight = options.rowHeight,
      renderRowContents = options.renderRowContents;
  var list = document.createElement('ul');
  list.classList.add('windowed');
  list.style.position = 'relative';
  setElHeight(items, rowHeight, list);

  var render = function render() {
    list.innerHTML = renderItems(items, renderRowContents, rowHeight, el);
  };

  var scrollHandler = optimiseHandler(render);
  window.addEventListener('scroll', scrollHandler);
  var resizeHandler = optimiseHandler(render);
  window.addEventListener('resize', resizeHandler);
  list.innerHTML = renderItems(items, renderRowContents, rowHeight, el);
  el.appendChild(list);
  return {
    setItems: function setItems(newItems) {
      items = newItems;
      setElHeight(items, rowHeight, list);
      render();
    },
    render: render,
    dispose: function dispose() {
      window.removeEventListener('scroll', scrollHandler);
      window.removeEventListener('resize', resizeHandler);
    }
  };
}

var _default = windowed;
exports.default = _default;